@@ -114,7 +114,10 @@ import Data.Enumerator
     )
 import qualified Data.Enumerator.List as EL
 import Network.HTTP.Enumerator.HttpParser
-import Control.Exception (Exception, bracket, throwIO, SomeException, try)
+import Control.Exception
+    ( Exception, bracket, throwIO, SomeException, try
+    , fromException
+    )
 import Control.Arrow (first)
 import Control.Monad.IO.Class (MonadIO (liftIO))
 import Control.Monad.Trans.Class (lift)
@@ -209,6 +212,13 @@ withSocketConn man host' port' =
     withManagedConn man (host', port', False) $
         fmap TLS.socketConn $ getSocket host' port'
 
+checkReset :: SomeException -> Bool
+checkReset e = isReset || isIOReset
+    where isReset = case fromException e of
+                      Just TLS.ConnectionReset -> True
+                      _ -> False
+          isIOReset = show e == "recv: resource vanished (Connection reset by peer)"
+
 withManagedConn
     :: MonadIO m
     => Manager
@@ -232,9 +242,12 @@ withManagedConn man key open req step = do
                 then putInsecureSocket man key ci
                 else TLS.connClose ci
             return a)
-        (\se -> if isManaged
-                    then withManagedConn man key open req step
-                    else throwError se)
+        (\se -> liftIO (TLS.connClose ci) >>
+                liftIO (putStrLn $ "Error during enum: " ++ show se) >>
+                if checkReset se && isManaged
+                   then withManagedConn man key open req step
+                   else throwError se
+        )
 
 withSslConn :: MonadIO m
             => ([X509] -> IO TLS.TLSCertificateUsage)
@@ -347,7 +360,7 @@ data RequestBody m
 -- | Add a Basic Auth header (with the specified user name and password) to the
 -- given Request. Ignore error handling:
 --
---    applyBasicAuth "user" "pass" $ fromJust $ parseUrl url
+--    applyBasicAuth \"user\" \"pass\" $ fromJust $ parseUrl url
 
 applyBasicAuth :: S.ByteString -> S.ByteString -> Request m -> Request m
 applyBasicAuth user passwd req =
@@ -596,7 +609,6 @@ encodeUrlChar c@'-' = [c]
 encodeUrlChar c@'_' = [c]
 encodeUrlChar c@'.' = [c]
 encodeUrlChar c@'~' = [c]
-encodeUrlChar ' ' = "+"
 encodeUrlChar y =
     let (a, c) = fromEnum y `divMod` 16
         b = a `mod` 16
@@ -726,7 +738,7 @@ httpLbs req = run_ . http req lbsIter
 -- | Download the specified URL, following any redirects, and return the
 -- response body.
 --
--- This function will 'failure' an 'HttpException' for any response with a
+-- This function will 'throwIO' an 'HttpException' for any response with a
 -- non-2xx status code. It uses 'parseUrl' to parse the input. This function
 -- essentially wraps 'httpLbsRedirect'.
 --
@@ -734,14 +746,14 @@ httpLbs req = run_ . http req lbsIter
 -- utilize lazy I/O, and therefore the entire response body will live in
 -- memory. If you want constant memory usage, you'll need to write your own
 -- iteratee and use 'http' or 'httpRedirect' directly.
-simpleHttp :: (MonadIO m, Failure HttpException m) => String -> m L.ByteString
+simpleHttp :: MonadIO m => String -> m L.ByteString
 simpleHttp url = do
-    url' <- parseUrl url
+    url' <- liftIO $ parseUrl url
     Response sc _ b <- liftIO $ withManager $ httpLbsRedirect
                                             $ url' { decompress = browserDecompress }
     if 200 <= sc && sc < 300
         then return b
-        else failure $ StatusCodeException sc b
+        else liftIO $ throwIO $ StatusCodeException sc b
 
 data HttpException = StatusCodeException Int L.ByteString
                    | InvalidUrlException String String
@@ -753,7 +765,7 @@ instance Exception HttpException
 -- | Same as 'http', but follows all 3xx redirect status codes that contain a
 -- location header.
 httpRedirect
-    :: (MonadIO m, Failure HttpException m)
+    :: MonadIO m
     => Request m
     -> (W.Status -> W.ResponseHeaders -> Iteratee S.ByteString m a)
     -> Manager
@@ -764,7 +776,7 @@ httpRedirect req bodyStep manager =
 -- | Make a request automatically follow 3xx redirects.
 --
 -- Used internally by 'httpRedirect' and family.
-redirectIter :: (MonadIO m, Failure HttpException m)
+redirectIter :: MonadIO m
              => Int -- ^ number of redirects to attempt
              -> Request m -- ^ Original request
              -> (W.Status -> W.ResponseHeaders -> Iteratee S.ByteString m a)
@@ -787,7 +799,7 @@ redirectIter redirects req bodyStep manager s@(W.Status code _) hs
                                 , S8.unpack l''
                                 ]
                             _ -> S8.unpack l''
-                l <- lift $ parseUrl l'
+                l <- liftIO $ parseUrl l'
                 let req' = req
                         { host = host l
                         , port = port l
@@ -800,7 +812,7 @@ redirectIter redirects req bodyStep manager s@(W.Status code _) hs
                                 else method l
                         }
                 if redirects == 0
-                    then lift $ failure TooManyRedirects
+                    then liftIO $ throwIO TooManyRedirects
                     else (http req') (redirectIter (redirects - 1) req' bodyStep manager) manager
             Nothing -> bodyStep s hs
     | otherwise = bodyStep s hs
@@ -822,7 +834,7 @@ redirectIter redirects req bodyStep manager s@(W.Status code _) hs
 -- /not/ utilize lazy I/O, and therefore the entire response body will live in
 -- memory. If you want constant memory usage, you'll need to write your own
 -- iteratee and use 'http' or 'httpRedirect' directly.
-httpLbsRedirect :: (MonadIO m, Failure HttpException m) => Request m -> Manager -> m Response
+httpLbsRedirect :: MonadIO m => Request m -> Manager -> m Response
 httpLbsRedirect req = run_ . httpRedirect req lbsIter
 
 readMay :: Read a => String -> Maybe a
